@@ -3,13 +3,13 @@ import numpy as np
 import tensorflow as tf
 
 from doctr import models
-from doctr.documents import read_pdf, Document
+from doctr.documents import Document, DocumentFile
 from test_models_detection import test_detectionpredictor
 from test_models_recognition import test_recognitionpredictor
 
 
 def test_extract_crops(mock_pdf):  # noqa: F811
-    doc_img = read_pdf(mock_pdf)[0]
+    doc_img = DocumentFile.from_pdf(mock_pdf).as_images()[0]
     num_crops = 2
     rel_boxes = np.array([[idx / num_crops, idx / num_crops, (idx + 1) / num_crops, (idx + 1) / num_crops]
                           for idx in range(num_crops)], dtype=np.float32)
@@ -39,9 +39,6 @@ def test_extract_crops(mock_pdf):  # noqa: F811
 
 def test_documentbuilder():
 
-    with pytest.raises(NotImplementedError):
-        models.DocumentBuilder(resolve_blocks=True)
-
     words_per_page = 10
     num_pages = 2
 
@@ -50,24 +47,24 @@ def test_documentbuilder():
     boxes = np.random.rand(words_per_page, 5)
     boxes[:2] *= boxes[2:4]
 
-    out = doc_builder([boxes, boxes], ['hello'] * (num_pages * words_per_page), [num_pages], [(100, 200), (100, 200)])
-    assert isinstance(out, list) and all(isinstance(doc, Document) for doc in out)
-    assert len(out[0].pages) == num_pages
+    out = doc_builder([boxes, boxes], ['hello'] * (num_pages * words_per_page), [(100, 200), (100, 200)])
+    assert isinstance(out, Document)
+    assert len(out.pages) == num_pages
     # 1 Block & 1 line per page
-    assert len(out[0].pages[0].blocks) == 1 and len(out[0].pages[0].blocks[0].lines) == 1
-    assert len(out[0].pages[0].blocks[0].lines[0].words) == words_per_page
+    assert len(out.pages[0].blocks) == 1 and len(out.pages[0].blocks[0].lines) == 1
+    assert len(out.pages[0].blocks[0].lines[0].words) == words_per_page
 
     # Resolve lines
-    doc_builder = models.DocumentBuilder(resolve_lines=True)
-    out = doc_builder([boxes, boxes], ['hello'] * (num_pages * words_per_page), [num_pages], [(100, 200), (100, 200)])
+    doc_builder = models.DocumentBuilder(resolve_lines=True, resolve_blocks=True)
+    out = doc_builder([boxes, boxes], ['hello'] * (num_pages * words_per_page), [(100, 200), (100, 200)])
 
     # No detection
     boxes = np.zeros((0, 5))
-    out = doc_builder([boxes, boxes], [], [num_pages], [(100, 200), (100, 200)])
-    assert len(out[0].pages[0].blocks) == 0
+    out = doc_builder([boxes, boxes], [], [(100, 200), (100, 200)])
+    assert len(out.pages[0].blocks) == 0
 
     # Repr
-    assert repr(doc_builder) == "DocumentBuilder(resolve_lines=True, paragraph_break=0.15)"
+    assert repr(doc_builder) == "DocumentBuilder(resolve_lines=True, resolve_blocks=True, paragraph_break=0.035)"
 
 
 @pytest.mark.parametrize(
@@ -76,7 +73,7 @@ def test_documentbuilder():
         [[[0, 0.5, 0.1, 0.6], [0, 0.3, 0.2, 0.4], [0, 0, 0.1, 0.1]], [2, 1, 0]],  # vertical
         [[[0.7, 0.5, 0.85, 0.6], [0.2, 0.3, 0.4, 0.4], [0, 0, 0.1, 0.1]], [2, 1, 0]],  # diagonal
         [[[0, 0.5, 0.1, 0.6], [0.15, 0.5, 0.25, 0.6], [0.5, 0.5, 0.6, 0.6]], [0, 1, 2]],  # same line, 2p
-        [[[0, 0.5, 0.1, 0.6], [0.2, 0.48, 0.35, 0.58], [0.8, 0.52, 0.9, 0.63]], [0, 1, 2]],  # ~same line
+        [[[0, 0.5, 0.1, 0.6], [0.2, 0.49, 0.35, 0.59], [0.8, 0.52, 0.9, 0.63]], [0, 1, 2]],  # ~same line
         [[[0, 0.3, 0.4, 0.45], [0.5, 0.28, 0.75, 0.42], [0, 0.45, 0.1, 0.55]], [0, 1, 2]],  # 2 lines
         [[[0, 0.3, 0.4, 0.35], [0.75, 0.28, 0.95, 0.42], [0, 0.45, 0.1, 0.55]], [0, 1, 2]],  # 2 lines
     ],
@@ -88,77 +85,63 @@ def test_sort_boxes(input_boxes, sorted_idxs):
 
 
 @pytest.mark.parametrize(
-    "input_boxes, sorted_idxs, lines",
+    "input_boxes, lines",
     [
-        [[[0, 0.5, 0.1, 0.6], [0, 0.3, 0.2, 0.4], [0, 0, 0.1, 0.1]], [2, 1, 0], [[2], [1], [0]]],  # vertical
-        [[[0.7, 0.5, 0.85, 0.6], [0.2, 0.3, 0.4, 0.4], [0, 0, 0.1, 0.1]], [2, 1, 0], [[2], [1], [0]]],  # diagonal
-        [[[0, 0.5, 0.1, 0.6], [0.15, 0.5, 0.25, 0.6], [0.5, 0.5, 0.6, 0.6]], [0, 1, 2], [[0, 1], [2]]],  # same line, 2p
-        [[[0, 0.5, 0.1, 0.6], [0.2, 0.48, 0.35, 0.58], [0.8, 0.52, 0.9, 0.63]], [0, 1, 2], [[0, 1], [2]]],  # ~same line
-        [[[0, 0.3, 0.4, 0.45], [0.5, 0.28, 0.75, 0.42], [0, 0.45, 0.1, 0.55]], [0, 1, 2], [[0, 1], [2]]],  # 2 lines
-        [[[0, 0.3, 0.4, 0.35], [0.75, 0.28, 0.95, 0.42], [0, 0.45, 0.1, 0.55]], [0, 1, 2], [[0], [1], [2]]],  # 2 lines
+        [[[0, 0.5, 0.1, 0.6], [0, 0.3, 0.2, 0.4], [0, 0, 0.1, 0.1]], [[2], [1], [0]]],  # vertical
+        [[[0.7, 0.5, 0.85, 0.6], [0.2, 0.3, 0.4, 0.4], [0, 0, 0.1, 0.1]], [[2], [1], [0]]],  # diagonal
+        [[[0, 0.5, 0.14, 0.6], [0.15, 0.5, 0.25, 0.6], [0.5, 0.5, 0.6, 0.6]], [[0, 1], [2]]],  # same line, 2p
+        [[[0, 0.5, 0.18, 0.6], [0.2, 0.48, 0.35, 0.58], [0.8, 0.52, 0.9, 0.63]], [[0, 1], [2]]],  # ~same line
+        [[[0, 0.3, 0.48, 0.45], [0.5, 0.28, 0.75, 0.42], [0, 0.45, 0.1, 0.55]], [[0, 1], [2]]],  # 2 lines
+        [[[0, 0.3, 0.4, 0.35], [0.75, 0.28, 0.95, 0.42], [0, 0.45, 0.1, 0.55]], [[0], [1], [2]]],  # 2 lines
     ],
 )
-def test_resolve_lines(input_boxes, sorted_idxs, lines):
+def test_resolve_lines(input_boxes, lines):
 
     doc_builder = models.DocumentBuilder()
-    assert doc_builder._resolve_lines(np.asarray(input_boxes), np.asarray(sorted_idxs)) == lines
+    assert doc_builder._resolve_lines(np.asarray(input_boxes)) == lines
 
 
 def test_ocrpredictor(mock_pdf, test_detectionpredictor, test_recognitionpredictor):  # noqa: F811
 
-    num_docs = 3
     predictor = models.OCRPredictor(
         test_detectionpredictor,
         test_recognitionpredictor
     )
 
-    docs = [read_pdf(mock_pdf) for _ in range(num_docs)]
-    out = predictor(docs)
+    doc = DocumentFile.from_pdf(mock_pdf).as_images()
+    out = predictor(doc)
 
-    assert len(out) == num_docs
     # Document
-    assert all(isinstance(doc, Document) for doc in out)
+    assert isinstance(out, Document)
     # The input PDF has 8 pages
-    assert all(len(doc.pages) == 8 for doc in out)
+    assert len(out.pages) == 8
     # Dimension check
     with pytest.raises(ValueError):
         input_page = (255 * np.random.rand(1, 256, 512, 3)).astype(np.uint8)
-        _ = predictor([[input_page]])
+        _ = predictor([input_page])
 
 
 @pytest.mark.parametrize(
-    "arch_name, top_implemented, input_shape, output_size",
+    "det_arch, reco_arch",
     [
-        ["vgg16_bn", False, (224, 224, 3), (7, 56, 512)],
-        ["resnet31", False, (32, 128, 3), (4, 32, 512)],
+        ["db_resnet50", "sar_vgg16_bn"],
+        ["db_resnet50", "crnn_vgg16_bn"],
+        ["db_resnet50", "sar_resnet31"],
+        ["db_resnet50", "crnn_resnet31"],
     ],
 )
-def test_classification_architectures(arch_name, top_implemented, input_shape, output_size):
-    # Head not implemented yet
-    if not top_implemented:
-        with pytest.raises(NotImplementedError):
-            models.__dict__[arch_name](include_top=True)
-
+def test_zoo_models(det_arch, reco_arch):
     # Model
-    batch_size = 2
-    model = models.__dict__[arch_name](pretrained=True)
-    # Forward
-    out = model(tf.random.uniform(shape=[batch_size, *input_shape], maxval=1, dtype=tf.float32))
-    # Output checks
-    assert isinstance(out, tf.Tensor)
-    assert out.numpy().shape == (batch_size, *output_size)
-
-
-@pytest.mark.parametrize(
-    "arch_name",
-    [
-        "ocr_db_sar_vgg",
-        "ocr_db_crnn_vgg",
-        "ocr_db_sar_resnet",
-    ],
-)
-def test_zoo_models(arch_name):
-    # Model
-    predictor = models.__dict__[arch_name](pretrained=True)
+    predictor = models.ocr_predictor(det_arch, reco_arch, pretrained=True)
     # Output checks
     assert isinstance(predictor, models.OCRPredictor)
+
+
+def test_preprocessor():
+    preprocessor = models.PreProcessor(output_size=(1024, 1024), batch_size=2)
+    input_tensor = tf.random.uniform(shape=[2, 512, 512, 3], minval=0, maxval=1)
+    preprocessed = preprocessor(input_tensor)
+    assert isinstance(preprocessed, list)
+    for batch in preprocessed:
+        assert batch.shape[0] == 2
+        assert batch.shape[1] == batch.shape[2] == 1024
