@@ -4,13 +4,13 @@ import numpy as np
 import tensorflow as tf
 
 from doctr.models import detection
-from doctr.documents import read_pdf
+from doctr.documents import DocumentFile
 
 
 def test_detpreprocessor(mock_pdf):  # noqa: F811
     num_docs = 3
     batch_size = 4
-    docs = [read_pdf(mock_pdf) for _ in range(num_docs)]
+    docs = [DocumentFile.from_pdf(mock_pdf).as_images() for _ in range(num_docs)]
     processor = detection.DetectionPreProcessor(output_size=(512, 512), batch_size=batch_size)
     batched_docs = processor([page for doc in docs for page in doc])
 
@@ -79,38 +79,38 @@ def test_dbpostprocessor():
     assert isinstance(out, tuple) and len(out) == 4
 
 
-def test_db_resnet50_training_mode():
-    model = detection.db_resnet50(pretrained=False)
-    assert isinstance(model, tf.keras.Model)
-    dbinput = tf.random.uniform(shape=[2, 1024, 1024, 3], minval=0, maxval=1)
-    # test training model
-    dboutput_train = model(dbinput, training=True)
-    assert isinstance(dboutput_train, tuple)
-    assert len(dboutput_train) == 3
-    assert all(np.all(np.logical_and(out_map.numpy() >= 0, out_map.numpy() <= 1)) for out_map in dboutput_train)
-    # batch size
-    assert all(out.numpy().shape == (2, 1024, 1024, 1) for out in dboutput_train)
-
-
 @pytest.mark.parametrize(
     "arch_name, input_shape, output_size, out_prob",
     [
         ["db_resnet50", (1024, 1024, 3), (1024, 1024, 1), True],
+        ["linknet", (1024, 1024, 3), (1024, 1024, 1), False],
     ],
 )
 def test_detection_models(arch_name, input_shape, output_size, out_prob):
-
     batch_size = 2
     model = detection.__dict__[arch_name](pretrained=True)
     assert isinstance(model, tf.keras.Model)
     input_tensor = tf.random.uniform(shape=[batch_size, *input_shape], minval=0, maxval=1)
-    # test prediction model
-    out = model(input_tensor)
-    assert isinstance(out, tf.Tensor)
-    out = out.numpy()
-    assert out.shape == (batch_size, *output_size)
+    target = [
+        dict(boxes=np.array([[0, 0, 1, 1], [0.5, 0.5, 1, 1]], dtype=np.float32), flags=[True, False]),
+        dict(boxes=np.array([[0, 0, 1, 1], [0.5, 0.5, 1, 1]], dtype=np.float32), flags=[True, False])
+    ]
+    # test training model
+    out = model(input_tensor, target, return_model_output=True, return_boxes=True, training=True)
+    assert isinstance(out, dict)
+    assert len(out) == 3
+    # Check proba map
+    seg_map = out['out_map'].numpy()
+    assert seg_map.shape == (batch_size, *output_size)
     if out_prob:
-        assert np.all(np.logical_and(out >= 0, out <= 1))
+        assert np.all(np.logical_and(seg_map >= 0, seg_map <= 1))
+    # Check boxes
+    for boxes in out['boxes']:
+        assert boxes.shape[1] == 5
+        assert np.all(boxes[:, :2] < boxes[:, 2:4])
+        assert np.all(boxes[:, :4] >= 0) and np.all(boxes[:, :4] <= 1)
+    # Check loss
+    assert isinstance(out['loss'], tf.Tensor)
 
 
 @pytest.fixture(scope="session")
@@ -119,11 +119,10 @@ def test_detectionpredictor(mock_pdf):  # noqa: F811
     batch_size = 4
     predictor = detection.DetectionPredictor(
         detection.DetectionPreProcessor(output_size=(512, 512), batch_size=batch_size),
-        detection.db_resnet50(input_shape=(512, 512, 3)),
-        detection.DBPostProcessor()
+        detection.db_resnet50(input_shape=(512, 512, 3))
     )
 
-    pages = read_pdf(mock_pdf)
+    pages = DocumentFile.from_pdf(mock_pdf).as_images()
     out = predictor(pages)
 
     # The input PDF has 8 pages
@@ -140,15 +139,34 @@ def test_detectionpredictor(mock_pdf):  # noqa: F811
 @pytest.mark.parametrize(
     "arch_name",
     [
-        "db_resnet50_predictor",
+        "db_resnet50",
+        "linknet",
     ],
 )
 def test_detection_zoo(arch_name):
     # Model
-    predictor = detection.zoo.__dict__[arch_name](pretrained=False)
+    predictor = detection.zoo.detection_predictor(arch_name, pretrained=False)
     # object check
     assert isinstance(predictor, detection.DetectionPredictor)
     input_tensor = tf.random.uniform(shape=[2, 1024, 1024, 3], minval=0, maxval=1)
     out = predictor(input_tensor)
     assert isinstance(out, list)
     assert all(isinstance(boxes, np.ndarray) and boxes.shape[1] == 5 for boxes in out)
+
+
+def test_detection_zoo_error():
+    with pytest.raises(ValueError):
+        _ = detection.zoo.detection_predictor("my_fancy_model", pretrained=False)
+
+
+def test_linknet_postprocessor():
+    postprocessor = detection.LinkNetPostProcessor()
+    mock_batch = tf.random.uniform(shape=[2, 512, 512, 1], minval=0, maxval=1)
+    out = postprocessor(mock_batch)
+    # Batch composition
+    assert isinstance(out, list)
+    assert len(out) == 2
+    assert all(isinstance(sample, np.ndarray) for sample in out)
+    assert all(sample.shape[1] == 5 for sample in out)
+    # Relative coords
+    assert all(np.all(np.logical_and(sample[:4] >= 0, sample[:4] <= 1)) for sample in out)
