@@ -4,14 +4,14 @@ import math
 import tensorflow as tf
 
 from doctr.models import recognition
-from doctr.documents import read_pdf
+from doctr.documents import DocumentFile
 from doctr.models import extract_crops
 
 
 def test_recopreprocessor(mock_pdf):  # noqa: F811
     num_docs = 3
     batch_size = 4
-    docs = [read_pdf(mock_pdf) for _ in range(num_docs)]
+    docs = [DocumentFile.from_pdf(mock_pdf).as_images() for _ in range(num_docs)]
     processor = recognition.RecognitionPreProcessor(output_size=(256, 128), batch_size=batch_size)
     batched_docs = processor([page for doc in docs for page in doc])
 
@@ -36,41 +36,28 @@ def test_recopreprocessor(mock_pdf):  # noqa: F811
 
 
 @pytest.mark.parametrize(
-    "arch_name, input_shape, output_size",
+    "arch_name, input_shape",
     [
-        ["crnn_vgg16_bn", (32, 128, 3), (32, 119)],
-        ["sar_vgg16_bn", (32, 128, 3), (31, 119)],
-        ["sar_resnet31", (32, 128, 3), (31, 119)],
-        ["crnn_resnet31", (32, 128, 3), (32, 119)],
+        ["crnn_vgg16_bn", (32, 128, 3)],
+        ["sar_vgg16_bn", (32, 128, 3)],
+        ["sar_resnet31", (32, 128, 3)],
+        ["crnn_resnet31", (32, 128, 3)],
     ],
 )
-def test_recognition_models(arch_name, input_shape, output_size):
-    batch_size = 8
-    reco_model = recognition.__dict__[arch_name](pretrained=True, input_shape=input_shape)
-    input_tensor = tf.random.uniform(shape=[batch_size, *input_shape], minval=0, maxval=1)
-    out = reco_model(input_tensor)
-    assert isinstance(out, tf.Tensor)
-    assert isinstance(reco_model, tf.keras.Model)
-    assert out.numpy().shape == (batch_size, *output_size)
-
-
-def test_sar_training():
+def test_recognition_models(arch_name, input_shape):
     batch_size = 4
-    input_shape = (32, 128, 3)
-    output_size = (31, 119)
-    reco_model = recognition.sar_vgg16_bn(input_shape=input_shape)
-    input_tensor = tf.random.uniform(shape=[batch_size, *input_shape], minval=0, maxval=1)
-    # input_labels: sparse_tensor of shape batch_size x max_len, encoding the labels
-    # filled with integers (classes of the characters at each timestep)
-    indices = [[0, 0], [0, 1], [1, 0], [1, 1], [1, 2], [2, 0], [3, 0], [3, 1], [3, 2], [3, 3], [3, 4]]
-    values = tf.random.uniform(shape=[11], minval=0, maxval=118, dtype=tf.dtypes.int64)
-    input_labels = tf.sparse.reorder(
-        tf.sparse.SparseTensor(indices=indices, values=values, dense_shape=[batch_size, 31])
-    )
-    out = reco_model(input_tensor, labels=input_labels, training=True)
-    assert isinstance(out, tf.Tensor)
+    reco_model = recognition.__dict__[arch_name](pretrained=True, input_shape=input_shape)
     assert isinstance(reco_model, tf.keras.Model)
-    assert out.numpy().shape == (batch_size, *output_size)
+    input_tensor = tf.random.uniform(shape=[batch_size, *input_shape], minval=0, maxval=1)
+    target = ["i", "am", "a", "jedi"]
+
+    out = reco_model(input_tensor, target, return_model_output=True, return_preds=True)
+    assert isinstance(out, dict)
+    assert len(out) == 3
+    assert isinstance(out['preds'], list)
+    assert len(out['preds']) == batch_size and all(isinstance(word, str) for word in out['preds'])
+    assert isinstance(out['out_map'], tf.Tensor)
+    assert isinstance(out['loss'], tf.Tensor)
 
 
 @pytest.mark.parametrize(
@@ -96,11 +83,10 @@ def test_recognitionpredictor(mock_pdf, mock_vocab):  # noqa: F811
     batch_size = 4
     predictor = recognition.RecognitionPredictor(
         recognition.RecognitionPreProcessor(output_size=(32, 128), batch_size=batch_size),
-        recognition.crnn_vgg16_bn(vocab_size=len(mock_vocab), input_shape=(32, 128, 3)),
-        recognition.CTCPostProcessor(mock_vocab)
+        recognition.crnn_vgg16_bn(vocab=mock_vocab, input_shape=(32, 128, 3))
     )
 
-    pages = read_pdf(mock_pdf)
+    pages = DocumentFile.from_pdf(mock_pdf).as_images()
     # Create bounding boxes
     boxes = np.array([[0, 0, 0.25, 0.25], [0.5, 0.5, 1., 1.]], dtype=np.float32)
     crops = extract_crops(pages[0], boxes)
@@ -122,19 +108,24 @@ def test_recognitionpredictor(mock_pdf, mock_vocab):  # noqa: F811
 @pytest.mark.parametrize(
     "arch_name",
     [
-        "crnn_vgg16_bn_predictor",
-        "sar_vgg16_bn_predictor",
-        "sar_resnet31_predictor",
-        "crnn_resnet31_predictor",
+        "crnn_vgg16_bn",
+        "sar_vgg16_bn",
+        "sar_resnet31",
+        "crnn_resnet31",
     ],
 )
 def test_recognition_zoo(arch_name):
     batch_size = 2
     # Model
-    predictor = recognition.zoo.__dict__[arch_name](pretrained=False)
+    predictor = recognition.zoo.recognition_predictor(arch_name, pretrained=False)
     # object check
     assert isinstance(predictor, recognition.RecognitionPredictor)
     input_tensor = tf.random.uniform(shape=[batch_size, 1024, 1024, 3], minval=0, maxval=1)
     out = predictor(input_tensor)
     assert isinstance(out, list) and len(out) == batch_size
     assert all(isinstance(word, str) for word in out)
+
+
+def test_recognition_zoo_error():
+    with pytest.raises(ValueError):
+        _ = recognition.zoo.recognition_predictor("my_fancy_model", pretrained=False)
